from .base import BaseScraper
from .config import config

class VideoScraper(BaseScraper):
    def scrape(self, endpoint_name: str, params: dict = None) -> str:
        endpoint = config.get_endpoint(endpoint_name, params.get('page') if params else None)
        if not endpoint:
            raise ValueError(f"Endpoint '{endpoint_name}' not found in configuration.")
        soup = self.get_soup(endpoint, params)
        return soup.prettify()
    
    



from bs4 import BeautifulSoup
import re
from .config import config

def get_videos(soup):
    return [parse_video(video) for video in soup.select('#content > .mozaique > .thumb-block') if parse_video(video) is not None]

def get_pages(soup, is_fresh):
    pages = set()
    for page in soup.select('.pagination > ul > li > a'):
        text = page.text.strip()
        if text.isdigit():
            page_number = int(text)
            pages.add(page_number - 1 if is_fresh else page_number)
    
    last_page = soup.select_one('.last-page')
    if last_page:
        text = last_page.text.strip()
        if text.isdigit():
            last_page_number = int(text)
            pages.add(last_page_number - 1 if is_fresh else last_page_number)

    return sorted(pages)

def parse_response(page, data, is_fresh=False):
    soup = BeautifulSoup(data, 'html.parser')
    videos = get_videos(soup)
    results_count = extract_results_count(soup, is_fresh)
    pages = get_pages(soup, is_fresh)
    last_page = pages[-1]
    
    if not is_fresh:
        last_page = 149 if int(results_count) >= 4029 else (results_count // 27) + 1
    
    pagination = {
        'page': page + 1,
        'pages': pages,
        'last_page': last_page
    }
    
    return {
        'videos': videos,
        'pagination': pagination,
        'has_next': page < last_page,
        'next': page + 1 if page < last_page else None,
        'has_previous': page > min(pages),
        'previous': page - 1 if page > min(pages) else None,
        'results_count': results_count,
    }

def extract_results_count(soup, is_fresh=False):
    if is_fresh: return 27 * 20000
    results_span = soup.select_one('h2.page-title .sub')
    if results_span:
        results_text = results_span.text.strip()
        match = re.search(r'\(([\d.,]+)', results_text)
        if match:
            results_count = int(match.group(1).replace('.', '').replace(',', ''))
        else:
            raise ValueError("No results found")
    else:
        raise ValueError("No results found")
    return results_count

def extract_title(video):
    title_element = video.select_one('.title a')
    if not title_element:
        raise ValueError("Title element not found")
    return title_element.get('title')

def extract_path_and_url(video):
    path_element = video.select_one('.thumb > a')
    if not path_element:
        raise ValueError("Path element not found")
    path = path_element.get('href')
    url = f"{config.get_base_url()}{path}"
    return path, url

def extract_duration(video):
    duration_element = video.select_one('.metadata .duration')
    return duration_element.text.strip() if duration_element else "Unknown"

def extract_profile(video):
    profile_element = video.select_one('.metadata span a')
    return {
        'name': profile_element.text.strip() if profile_element else "Unknown",
        'url': f"{config.get_base_url()}{profile_element.get('href')}" if profile_element else "Unknown"
    }

def extract_views(video):
    views = "Unknown"
    metadata = video.select_one('.metadata')
    if metadata:
        sprfluous_spans = metadata.find_all('span', class_='sprfluous')
        if len(sprfluous_spans) >= 2:
            views_text = sprfluous_spans[0].find_next_sibling(text=True)
            if views_text:
                views = views_text.strip().split()[0]
    return views

def parse_video(video):
    try:
        title = extract_title(video)
        path, url = extract_path_and_url(video)
        duration = extract_duration(video)
        profile = extract_profile(video)
        views = extract_views(video)
        return {
            'url': url,
            'path': path,
            'title': title,
            'duration': duration,
            'profile': profile,
            'views': views,
        }
    except ValueError as e:
        print(f"Error parsing video: {e}")
        return None



from bs4 import BeautifulSoup;
import requests
from abc import ABC, abstractmethod
from .config import config
class BaseScraper():
    def __init__(self):
        self.base_url = config.get_base_url()
    @abstractmethod
    def scrape(self, endpoint: str, params: dict) -> str:
        pass
    def get_soup(self, endpoint: str, params: dict) -> BeautifulSoup:
        response = requests.get(f"{self.base_url}{endpoint}", params=params)
        return BeautifulSoup(response.text, 'html.parser')


from .base import VideoScraper, parse_response

class FreshScraper(VideoScraper):
    def fresh(self, page: int = 1) -> str:
        if page == 0:
            endpoint = "fresh"
        else:
            endpoint = 'fresh_p'
        data = self.scrape(endpoint, {'page': page} if page else None)
        return parse_response(page, data, is_fresh=True)


from .base import VideoScraper, parse_response

class VerifiedScraper(VideoScraper):
    def get_verified(self, page: int = 1, type="women") -> str:
        if type == "women":
            endpoint = "verified"
        elif type == "men":
            endpoint = "verified_men"
        elif type == "gay":
            endpoint = f'verified_gay'
        elif type == "trans":
            endpoint = f'verified_trans'
        else:
            raise ValueError("Endpoint must be either women, men, gay or trans")
        if page <= 0: endpoint += '_p'
        data = self.scrape(endpoint, {'page': page} if page else None)
        return parse_response(page, data, is_fresh=True)


from .xvideos import XVideos
def main():
    xvideos = XVideos()
    print(xvideos.fresh(page=1)) 
    print(xvideos.search(page=1, k="example")) 
    print(xvideos.get_verified()) 

if __name__ == "__main__":
    main()



import pytest
from xvideos.xvideos import XVideos

@pytest.fixture
def scraper():
    return XVideos()
    
def test_fresh(scraper):
    result = scraper.fresh(page=2)
    assert 'pagination' in result

def test_search(scraper):
    result = scraper.search(page=2, k="example", sort="relevance")
    assert 'pagination' in result

def test_get_verified(scraper):
    result = scraper.get_verified(page=2)
    assert 'videos' in result
    assert len(result['videos']) > 0

def test_fresh_with_negative_page(scraper):
    with pytest.raises(ValueError, match="Page must be an integer greater than 0"):
        scraper.fresh(page=-1)

def test_search_with_negative_page(scraper):
    with pytest.raises(ValueError, match="Page must be an integer greater than 0"):
        scraper.search(page=-1, k="example", sort="relevance")

def test_get_verified_with_negative_page(scraper):
    with pytest.raises(ValueError, match="Page must be an integer greater than 0"):
        scraper.get_verified(page=-1)

def test_search_with_empty_string_params(scraper):
    with pytest.raises(ValueError):
        scraper.search(page=1, k="", sort="", durf="", datef="", quality="", premium=False)

def test_fresh_with_large_page(scraper):
    with pytest.raises(ValueError):
        scraper.fresh(page=99999)

def test_search_with_large_page(scraper):
    result = scraper.search(page=99999, k="example", sort="relevance")
    assert 'pagination' in result

def test_get_verified_with_large_page(scraper):
    with pytest.raises(ValueError):
        scraper.get_verified(page=99999)
        
def test_search_without_page(scraper):
    result = scraper.search(k="example", sort="relevance")
    assert 'pagination' in result





from .videos import FreshScraper, SearchScraper, VerifiedScraper

    
def page_validator(func):
    def wrapper(self, page=1, *args, **kwargs):
        if page <= 0:
            raise ValueError("Page must be an integer greater than 0")
        page -= 1
        return func(self, page, *args, **kwargs)
    return wrapper

SAFE_MAX_NUMBER_VERIFIED = 254
SAFE_MAX_NUMBER_FRESH = 20000
class XVideos():
    def __init__(self):
        self.__fresh_scraper = FreshScraper()
        self.__search_scraper = SearchScraper()
        self.__verified_scraper = VerifiedScraper()

    @page_validator
    def fresh(self, page=1):
        if page >= SAFE_MAX_NUMBER_FRESH: raise ValueError(f"Page parameter must be under {SAFE_MAX_NUMBER_FRESH}")
        return self.__fresh_scraper.fresh(page)

    @page_validator
    def search(self, page=1, k="", sort="relevance", durf="allduration", datef="all", quality="all", premium=False):
        return self.__search_scraper.search(page, k, sort, durf, datef, quality, premium)

    @page_validator
    def get_verified(self, page=1):
        if page >= SAFE_MAX_NUMBER_VERIFIED: raise ValueError(f"Page parameter must be under {SAFE_MAX_NUMBER_VERIFIED}")
        return self.__verified_scraper.get_verified(page)


        


import json
import os

class Config:
    def __init__(self, config_file: str):
        with open(config_file, 'r') as file:
            config_data = json.load(file)
        
        self.base_url = config_data['base_url']
        self.endpoints = config_data['endpoints']

    def get_base_url(self):
        return self.base_url

    def get_endpoint(self, endpoint_name: str, page: int = None) -> str:
        endpoint = self.endpoints.get(endpoint_name, '')
        if page is not None:
            return endpoint.format(page=page)
        else:
            return endpoint

config = Config(os.path.join(os.path.dirname(__file__), 'config.json'))




from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="xvideos-py",
    version="0.1.3",
    description="A powerful Python library to scrape xvideos.com",
    author="OSintt",
    author_email="akumagamer123@gmail.com",
    url="https://github.com/OSintt/xvideos.py",
    packages=find_packages(),
    package_data={
        'xvideos': ['videos/base/config/config.json'],
    },
    install_requires=[
        "beautifulsoup4",
        "requests",
    ],
    tests_require=[
        "pytest",
    ],
    entry_points={
        'console_scripts': [
            'xvideos=xvideos.main:main',
        ],
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    long_description=long_description,
    long_description_content_type="text/markdown",
)



from .base import VideoScraper, parse_response
class SearchScraper(VideoScraper):
    def search(
        self, page: int = 1, k: str = "", sort: str = "relevance", durf: str = "allduration", datef: str = "all", quality: str = "all", premium: bool = False
    ) -> str:
        params = {
            'p': page,
            'k': k,
            'sort': sort,
            'durf': durf,
            'datef': datef,
            'quality': quality,
            'premium': int(premium)
        }
        data = self.scrape('search', params)
        return parse_response(page, data, is_fresh=False)

